#! /usr/bin/python -O
# -*- coding: iso-8859-15 -*-
#
# Lode Runner level 2 ascii art.
# Copyright (C) 2012 MiKael NAVARRO
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""Lode Runner level 2 ascii art.
Copyright (C) 2012 MiKael NAVARRO

Display 'bitmap' level to ascii art.

"""

# Specific variables for pydoc
__author__ = "MiKael Navarro <klnavarro@gmail.com>"
__date__ = "Wed Dec 19 2012"
__version__ = "0.1.0"
__credits__ = """Thanks to Andrew Schultz <http://webspace.webring.com/people/rd/drkerwood/loderunner/main.htm>."""

# Include directives
import sys
import os
from string import maketrans  # translation rules

# Global variables
TRANSLATE_TBL1 = maketrans("0123456789", " X*H~Vh$-+")
TRANSLATE_TBL2 = maketrans("0123456789", " #*H~Vh$g")


#
# Decode bitmap to ascii.
#
def lvl2asc1(bmap, name):
    """Translate bitmap to ascii (drkerwood)."""
    
    asc = ""
    for l in bmap:
        if l.startswith('d28x16'):
            asc += "// %s (%s)\n" % (name, l.strip())
        else:
            asc += l.strip().translate(TRANSLATE_TBL1) + '\n'
    return asc

def lvl2asc2(bmap, name):
    """Translate bitmap to ascii (klnavarro)."""
    
    asc = ""
    for l in bmap:
        if l.startswith('d28x16'):
            asc += "// %s (%s)\n" % (name, l.strip())
        else:
            asc += l.strip().translate(TRANSLATE_TBL2) + '\n'
    return asc


#
# Main entry point.
#
def main(fname, asctype):
    """Parse filename, then convert to ascii art."""

    f = open(fname, 'r')
    try:
        lvl_name = os.path.basename(fname.rstrip('.txt'))  # level name
        
        if asctype == 'drkerwood':
            print lvl2asc1(f.readlines(), lvl_name)
            
        elif asctype == 'klnavarro':
            print lvl2asc2(f.readlines(), lvl_name)

    finally:
        f.close()     


#
# External entry point.
#
if __name__ == "__main__":
    # Get options
    from optparse import OptionParser, make_option

    option_list = [
        make_option("-a", "--asc",
                    action="store", dest="asctype",
                    default="klnavarro",
                    choices=["drkerwood", "klnavarro"],
                    help="define output format (default='klnavarro')"),
        ]

    parser = OptionParser(usage="python -O %prog [options] level.txt", 
                          version=__version__,
                          description="Display Lode Runner level file as ascii art.",
                          option_list=option_list)

    (options, args) = parser.parse_args()

    # Needed argument
    if not args:
        parser.error("missing level.txt file argument")
    elif len(args) != 1:
        parser.error("too many arguments")


    # Process start here
    main(args[0], options.asctype)
