#! /usr/bin/python -O
# -*- coding: iso-8859-15 -*-
#
# Lode Runner level 2 JS.
# Copyright (C) 2012 MiKael NAVARRO
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""Lode Runner level 2 JS.
Copyright (C) 2012 MiKael NAVARRO

Convert level files to JS array.

"""

# Specific variables for pydoc
__author__ = "MiKael Navarro <klnavarro@gmail.com>"
__date__ = "Wed Dec 19 2012"
__version__ = "0.1.0"
__credits__ = """Thanks to Andrew Schultz <http://webspace.webring.com/people/rd/drkerwood/loderunner/main.htm>."""

# Include directives
import sys
import os
from glob import glob


#
# Transform bitmap to JS array.
#
def lvl2js(bmap, lvl):
    """Transform 'bitmap' to JS array."""

    # JS array of array
    js = "MINE_LEVELS[%d] = [];\n" % (lvl,)
    for i, l in enumerate(bmap):
        js += "MINE_LEVELS[%d][%d] = [];\n" % (lvl, i)
        for j, c in enumerate(l.strip()):
            js += "MINE_LEVELS[%d][%d][%d] = %s;\n" % (lvl, i, j, c)
    return js


#
# Main entry point.
#
def main(fnames, options):
    """Parse filenames, then convert bitmap to JS array."""

    pretty = not options.compress

    # Initialize JS array
    print "var MINE_LEVELS = [];"

    # Add levels...
    for fname in fnames:
        f = open(fname, 'r')
        try:
            lvl = os.path.basename(fname.rstrip('.txt').replace("level", "", 1))  # level num
            lvls = lvl2js(f.readlines()[1:], int(lvl))  # skip first line (d28x16)
            if pretty: print lvls,
            else: print lvls.translate(None, " \n")

        finally:
            f.close()

    # Exports
    print "exports.MINE_LEVELS = MINE_LEVELS;"


#
# External entry point.
#
if __name__ == "__main__":
    # Get options
    from optparse import OptionParser, make_option

    option_list = [
        #make_option("-o", "--output",
        #            action="store", dest="output", 
        #            type="string", default="levels.js",
        #            help="output filename"),
        make_option("-c", "--compress",
                    action="store_true", dest="compress", 
                    default=False,
                    help="compress generated JS file"),
        ]

    parser = OptionParser(usage="python -O %prog level1.txt[, ...]", 
                          version=__version__,
                          description="Convert Lode Runner levels to JS array.",
                          option_list=option_list)

    (options, args) = parser.parse_args()

    # Needed argument
    if not args:
        parser.error("missing level file argument")
    else:
        files = []
        for fn in args:
            files.extend(glob(fn))

    # Process start here
    main(files, options)
