/*
 * LRJS - Simple Lode Runner clone.
 * Copyright (C) 2014 MiKael NAVARRO
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Specific variables for pydoc */
var AUTHOR = "MiKael Navarro <klnavarro@gmail.com>"
var DATE = "Wed Apr 02 2014"
var VERSION = "20140402"
var CREDITS = "Thanks to Andrew SCHULTZ <http://webspace.webring.com/people/rd/drkerwood/loderunner/main.htm> for these links to all levels of Lode Runner."

/* Include directives */
var gamejs = require('gamejs');
var MINE_LEVELS = require('levels').MINE_LEVELS;

/* Extends GameJS keys events */
gamejs.event.K_INS = 45;
gamejs.event.K_DEL = 46;
gamejs.event.K_END = 35;
gamejs.event.K_HOME = 36;
gamejs.event.K_PUP = 33;
gamejs.event.K_PDOWN = 34;

/* Global variables */
var BLANK_SPACE = 0;
var BRICK = 1;
var CEMENT = 2;
var LADDER = 3;
var ROPE_LADDER = 4;
var FALSE_BRICK = 5;
var LADDER_EXIT = 6;
var GOLD_CHEST = 7;
var ROBOT = 8;
var LODE_RUNNER = 9;
var HOLE = 'A';
var HOLE0 = 'B';
var HOLE1 = 'C';
var HOLE2 = 'D';

var SPRITE_WIDTH = 10;
var SPRITE_HEIGHT = 11;

var PLAYER = { 'anim': 0,
               'w': SPRITE_WIDTH,
               'h': SPRITE_HEIGHT,
               'c': 0,
               'l': 0,
               'x': this.c*SPRITE_WIDTH,
               'y': this.l*SPRITE_HEIGHT,
               'v': 8,  /* 8 px/s */
               'pos': 'wait0',
               'dir': 'none',
               'sprites': {
                   'wait0': { 'x': 9*SPRITE_WIDTH, 'y': 0*SPRITE_HEIGHT },
                   'wait1': { 'x': 9*SPRITE_WIDTH, 'y': 0*SPRITE_HEIGHT },
                   'walk_right0': { 'x': 0*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'walk_right1': { 'x': 1*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'walk_right2': { 'x': 2*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'walk_left0': { 'x': 5*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'walk_left1': { 'x': 4*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'walk_left2': { 'x': 3*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'climb0': { 'x': 6*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'climb1': { 'x': 7*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'rope_right0': { 'x': 8*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'rope_right1': { 'x': 9*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'rope_right2': { 'x': 10*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'rope_left0': { 'x': 13*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'rope_left1': { 'x': 12*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'rope_left2': { 'x': 11*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'fall_right': { 'x': 14*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'fall_left': { 'x': 15*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'shoot_right': { 'x': 16*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT },
                   'shoot_left': { 'x': 17*SPRITE_WIDTH, 'y': 1*SPRITE_HEIGHT }
               },
               'transition': {
                   'wait0': { 'left': 'walk_left0', 'right': 'walk_right0',
                              'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                              'none': 'wait1',
                              'up': 'climb1', 'down': 'climb1', 'fall': 'fall_right' },
                   'wait1': { 'left': 'walk_left0', 'right': 'walk_right0',
                              'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                              'none': 'wait0',
                              'up': 'climb0', 'down': 'climb0', 'fall': 'fall_left' },
                   'walk_right0': { 'left': 'walk_left0', 'right': 'walk_right1',
                                    'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                    'none': 'walk_right0',
                                    'up': 'climb0', 'down': 'climb0', 'fall': 'fall_right' },
                   'walk_right1': { 'left': 'walk_left0', 'right': 'walk_right2',
                                    'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                    'none': 'walk_right1',
                                    'up': 'climb0', 'down': 'climb0', 'fall': 'fall_right' },
                   'walk_right2': { 'left': 'walk_left0', 'right': 'walk_right0',
                                    'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                    'none': 'walk_right2',
                                    'up': 'climb0', 'down': 'climb0', 'fall': 'fall_right' },
                   'walk_left0': { 'left': 'walk_left1', 'right': 'walk_right0',
                                   'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                   'none': 'walk_left0',
                                   'up': 'climb1', 'down': 'climb1', 'fall': 'fall_left' },
                   'walk_left1': { 'left': 'walk_left2', 'right': 'walk_right0',
                                   'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                   'none': 'walk_left1',
                                   'up': 'climb1', 'down': 'climb1', 'fall': 'fall_left' },
                   'walk_left2': { 'left': 'walk_left0', 'right': 'walk_right0',
                                   'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                   'none': 'walk_left2',
                                   'up': 'climb1', 'down': 'climb1', 'fall': 'fall_left' },
                   'climb0': { 'left': 'walk_left0', 'right': 'walk_right0',
                               'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                               'none': 'climb0',
                               'up': 'climb1', 'down': 'climb1', 'fall': 'fall_left' },
                   'climb1': { 'left': 'walk_left0', 'right': 'walk_right0',
                               'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                               'none': 'climb1',
                               'up': 'climb0', 'down': 'climb0', 'fall': 'fall_right' },
                   'rope_right0': { 'left': 'walk_left0', 'right': 'walk_right1',
                                    'rope_left': 'rope_left0', 'rope_right': 'rope_right1',
                                    'none': 'rope_right0',
                                    'up': 'climb0', 'down': 'fall_right', 'fall': 'fall_right' },
                   'rope_right1': { 'left': 'walk_left0', 'right': 'walk_right2',
                                    'rope_left': 'rope_left0', 'rope_right': 'rope_right2',
                                    'none': 'rope_right1',
                                    'up': 'climb0', 'down': 'fall_right', 'fall': 'fall_right' },
                   'rope_right2': { 'left': 'walk_left0', 'right': 'walk_right0',
                                    'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                    'none': 'rope_right2',
                                    'up': 'climb0', 'down': 'fall_right', 'fall': 'fall_right' },
                   'rope_left0': { 'left': 'walk_left1', 'right': 'walk_right0',
                                   'rope_left': 'rope_left1', 'rope_right': 'rope_right0',
                                   'none': 'rope_left0',
                                   'up': 'climb1', 'down': 'fall_left', 'fall': 'fall_left' },
                   'rope_left1': { 'left': 'walk_left2', 'right': 'walk_right0',
                                   'rope_left': 'rope_left2', 'rope_right': 'rope_right0',
                                   'none': 'rope_left1',
                                   'up': 'climb1', 'down': 'fall_left', 'fall': 'fall_left' },
                   'rope_left2': { 'left': 'walk_left0', 'right': 'walk_right0',
                                   'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                   'none': 'rope_left2',
                                   'up': 'climb1', 'down': 'fall_left', 'fall': 'fall_left' },
                   'fall_right': { 'left': 'walk_left0', 'right': 'walk_right0',
                                   'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                   'none': 'fall_right',
                                   'up': 'climb1', 'down': 'climb1', 'fall': 'fall_right' },
                   'fall_left': { 'left': 'walk_left0', 'right': 'walk_right0',
                                  'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                  'none': 'fall_left',
                                  'up': 'climb0', 'down': 'climb0', 'fall': 'fall_left' },
                   'shoot_right': { 'left': 'walk_left0', 'right': 'walk_right0',
                                    'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                    'none': 'shoot_right',
                                    'up': 'climb1', 'down': 'climb1', 'fall': 'fall_right' },
                   'shoot_left': { 'left': 'walk_left0', 'right': 'walk_right0',
                                   'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                   'none': 'climb1',
                                   'up': 'climb0', 'down': 'climb0', 'fall': 'fall_left' }
               },
               'keys': [],
               'input': function(event) {
                   /* Keyboard inputs */
                   //gamejs.log(event.type + "/" + event.key);
                   this.keys[event.key] = event.type == gamejs.event.KEY_DOWN;
                   /* Player direction */
                   if (this.keys[gamejs.event.K_UP] || PLAYER.keys[gamejs.event.K_i]) {
                       this.dir = 'up';
                       MINE.paused = false;
                       MINE.state = IN_PROGRESS;
                   }
                   else if (this.keys[gamejs.event.K_DOWN] || this.keys[gamejs.event.K_k]) {
                       this.dir = 'down';
                       MINE.paused = false;
                       MINE.state = IN_PROGRESS;
                   }
                   else if (this.keys[gamejs.event.K_RIGHT] || this.keys[gamejs.event.K_l]) {
                       this.dir = 'right';
                       MINE.paused = false;
                       MINE.state = IN_PROGRESS;
                   }
                   else if (this.keys[gamejs.event.K_LEFT] || this.keys[gamejs.event.K_j]) {
                       this.dir = 'left';
                       MINE.paused = false;
                       MINE.state = IN_PROGRESS;
                   }
                   else if (this.keys[gamejs.event.K_x] || this.keys[gamejs.event.K_u]) {
                       this.dir = 'dig_left';
                       MINE.paused = false;
                       MINE.state = IN_PROGRESS;
                   }
                   else if (this.keys[gamejs.event.K_c] || this.keys[gamejs.event.K_o]) {
                       this.dir = 'dig_right';
                       MINE.paused = false;
                       MINE.state = IN_PROGRESS;
                   }
                   else {
                       this.dir = 'none';
                   }
                   /* Game state */
                   if (this.keys[gamejs.event.K_p]) {
                       /* Pause game */
                       MINE.paused = !MINE.paused;
                   }
                   if (this.keys[gamejs.event.K_s]) {
                       /* Switch sound on/off */
                       gamejs.log("sound on/off");
                   }
                   if (this.keys[gamejs.event.K_DEL] || this.keys[gamejs.event.K_ESC]) {
                       /* Reinit level */
                       MINE.map = _.cloneDeep(MINE_LEVELS[MINE.lvl]);
                       $("#level").text(("000" + MINE.lvl).substr(-3));
                       ROBOTS = [];  /* reinit */
                       MINE.paused = true;
                       MINE.nb_robots = 0;
                       MINE.gold = 0;
                       this.anim = 0;
                       /* Reload score from cookie */
                       var lrjs_data = $.cookie('lrjs').split(',');
                       this.score = +lrjs_data[0];
                       if (this.men > 0) {  /* kill man */
                           this.men -= 1; 
                           $("#men").text(("000" + this.men).substr(-3));
                       }
                       $("#score").text(("0000000" + this.score).substr(-7));
                       $("#men").text(("000" + this.men).substr(-3));
                       /* Save to cookie */
                       $.cookie('lrjs', ("0000000"+this.score).substr(-7)+","+("000"+this.men).substr(-3)+","+("000"+MINE.lvl).substr(-3), { expires: 365 });
                   }
                   /* "cheats" controls */
                   if (this.keys[gamejs.event.K_INS] && this.men < 999) {
                       /* Add extra man */
                       this.men += 1;
                       $("#men").text(("000" + this.men).substr(-3));
                       /* Save to cookie */
                       $.cookie('lrjs', ("0000000"+this.score).substr(-7)+","+("000"+this.men).substr(-3)+","+("000"+MINE.lvl).substr(-3), { expires: 365 });
                   }
                   if (this.keys[gamejs.event.K_PUP] && MINE.lvl < MINE_LEVELS.length-1) {
                       /* Skip current level, go next */
                       MINE.lvl = +MINE.lvl + 1;
                       MINE.map = _.cloneDeep(MINE_LEVELS[MINE.lvl]);
                       $("#level").text(("000" + MINE.lvl).substr(-3));
                       ROBOTS = [];  /* reinit */
                       MINE.paused = true;
                       MINE.nb_robots = 0;
                       MINE.gold = 0;
                       this.anim = 0;
                       /* Reload score & lives from cookie */
                       var lrjs_data = $.cookie('lrjs').split(',');
                       this.score = +lrjs_data[0];
                       this.men = +lrjs_data[1];
                       $("#score").text(("0000000" + this.score).substr(-7));
                       $("#men").text(("000" + this.men).substr(-3));
                       /* Save to cookie */
                       $.cookie('lrjs', ("0000000"+this.score).substr(-7)+","+("000"+this.men).substr(-3)+","+("000"+MINE.lvl).substr(-3), { expires: 365 });
                   }
                   if (this.keys[gamejs.event.K_PDOWN] && MINE.lvl > 1) {
                       /* Go previous level */
                       MINE.lvl = +MINE.lvl - 1;
                       MINE.map = _.cloneDeep(MINE_LEVELS[MINE.lvl]);
                       $("#level").text(("000" + MINE.lvl).substr(-3));
                       ROBOTS = [];  /* reinit */
                       MINE.paused = true;
                       MINE.nb_robots = 0;
                       MINE.gold = 0;
                       this.anim = 0;
                       /* Reload score & lives from cookie */
                       var lrjs_data = $.cookie('lrjs').split(',');
                       this.score = +lrjs_data[0];
                       this.men = +lrjs_data[1];
                       $("#score").text(("0000000" + this.score).substr(-7));
                       $("#men").text(("000" + this.men).substr(-3));
                       /* Save to cookie */
                       $.cookie('lrjs', ("0000000"+this.score).substr(-7)+","+("000"+this.men).substr(-3)+","+("000"+MINE.lvl).substr(-3), { expires: 365 });
                   }
                   //gamejs.log(this.keys);
               },
               'men': 1,
               'score': 0,
               'update': function(msDuration) {
                   /* Updt animation frame */
                   this.anim += msDuration;

                   $("#anim").text(("000" + this.anim).substr(-3));
                   $("#px").text(this.x);
                   $("#py").text(this.y);
                   $("#pc").text(this.c);
                   $("#pl").text(this.l);

                   /* Fall down? */
                   if (this.anim > 1000/this.v) {
                       /* Current pos/floor */
                       var pc = this.c;
                       var pl = this.l;

                       if ( this.l < MINE.lines-1
                            && MINE.map[this.l][this.c] != LADDER
                            && MINE.map[this.l][this.c] != ROPE_LADDER
                            && MINE.map[this.l+1][this.c] != BRICK
                            && MINE.map[this.l+1][this.c] != CEMENT 
                            && MINE.map[this.l+1][this.c] != LADDER ) {
                           /* Collision detection */
                           var collide = ROBOTS.some(function(r) {
                               return (pc == r.c && (pl+1) == r.l);
                           });
                           if (!collide) {
                               this.dir = 'fall';  // fall!!!
                           } 
                       }

                       /* Update player position */
                       $("#dir").text(this.dir);
                       switch(this.dir) {
                       case 'left':
                       case 'walk_left':
                       case 'rope_left':
                           /* Can I move left? */
                           if ( this.c-1 < 0
                                || MINE.map[this.l][this.c-1] == BRICK
                                || MINE.map[this.l][this.c-1] == CEMENT ) {
                               this.dir = 'none';  /* no */
                           }
                           else if ( MINE.map[this.l][this.c-1] == ROPE_LADDER ) {
                               this.dir = 'rope_left';
                           }
                           /* Ok, move left */
                           if (this.dir != 'none') {
                               this.c -= 1;
                           }
                           /* Updt animation frame */
                           this.pos = this.transition[this.pos][this.dir];
                           break;
                       case 'right':
                       case 'walk_right':
                       case 'rope_right':
                           /* Can I move right? */
                           if ( this.c+1 > MINE.cols-1
                                || MINE.map[this.l][this.c+1] == BRICK
                                || MINE.map[this.l][this.c+1] == CEMENT ) {
                               this.dir = 'none';  /* no */
                           }
                           else if ( MINE.map[this.l][this.c+1] == ROPE_LADDER ) {
                               this.dir = 'rope_right';
                           }
                           /* Ok, move right */
                           if (this.dir != 'none') {
                               this.c += 1;
                           }
                           /* Updt animation frame */
                           this.pos = this.transition[this.pos][this.dir];
                           break;
                       case 'up':
                           /* Can I move up? */
                           if ( this.l == 0 
                                && MINE.map[this.l][this.c] == LADDER ) {
                               this.l -= 1;
                               return;  /* exit */
                           }
                           if ( this.l-1 < 0 
                                || MINE.map[this.l][this.c] != LADDER
                                || MINE.map[this.l-1][this.c] == BRICK
                                || MINE.map[this.l-1][this.c] == CEMENT ) {
                               this.dir = 'none';;
                           }
                           /* Ok, move up */
                           if (this.dir != 'none') {
                               this.l -= 1;
                           }
                           /* Updt animation frame */
                           this.pos = this.transition[this.pos][this.dir];
                           break;
                       case 'down':
                           /* Can I move down? */
                           if ( this.l+1 > MINE.lines-1
                                || MINE.map[this.l+1][this.c] == BRICK
                                || MINE.map[this.l+1][this.c] == CEMENT ) {
                               this.dir = 'none';
                           }
                           /* Ok, move down */
                           if (this.dir != 'none') {
                               this.l += 1;
                           }
                           /* Updt animation frame */
                           this.pos = this.transition[this.pos][this.dir];
                           break;
                       case 'fall':
                           gamejs.log("fall");
                           /* Fall down? */
                           if ( this.l+1 > MINE.lines-1
                                || MINE.map[this.l][this.c] == LADDER
                                || MINE.map[this.l][this.c] == ROPE_LADDER
                                || MINE.map[this.l+1][this.c] == BRICK
                                || MINE.map[this.l+1][this.c] == CEMENT 
                                || MINE.map[this.l+1][this.c] == LADDER ) {
                               this.dir = 'none';
                           }
                           /* Allow to walk on the heads of robots ;) */
                           var collide = ROBOTS.some(function(r) {
                               return (pc == r.c && (pl+1) == r.l);
                           });
                           if (collide) {
                               this.dir = 'none';
                           } 
                           /* Ok, fall down */
                           if (this.dir != 'none') {
                               this.l += 1;
                           }
                           /* Updt animation frame */
                           this.pos = this.transition[this.pos][this.dir];
                           break;
                       case 'dig_left':
                           if ( this.c-1 >= 0 && this.l+1 <= MINE.lines-1
                                && MINE.map[this.l+1][this.c-1] == BRICK 
                                && MINE.map[this.l][this.c-1] != BRICK 
                                && MINE.map[this.l][this.c-1] != CEMENT
                                && MINE.map[this.l][this.c-1] != LADDER ) {
                               MINE.map[this.l+1][this.c-1] = HOLE;
                           }
                           break;
                       case 'dig_right':
                           if ( this.c+1 <= MINE.cols-1 && this.l+1 <= MINE.lines-1
                                && MINE.map[this.l+1][this.c+1] == BRICK 
                                && MINE.map[this.l][this.c+1] != BRICK
                                && MINE.map[this.l][this.c+1] != CEMENT
                                && MINE.map[this.l][this.c+1] != LADDER ) {
                               MINE.map[this.l+1][this.c+1] = HOLE;
                           }
                           break;
                       }
                   }
                   $("#pos").text(this.pos);
                   if (this.anim > 1000/this.v) {
                       this.anim = this.anim % (1000/this.v) + 1;  /* reset current animation counter */
                   }
                   
                   /* Gold chest? */
                   if ( MINE.map[this.l][this.c] == GOLD_CHEST ) {
                       this.score += 250;
                       $("#score").text(("0000000" + this.score).substr(-7));
                       MINE.map[this.l][this.c] = BLANK_SPACE;
                       MINE.gold -= 1;
                       $("#gold").text(MINE.gold);
                   }
               },
               'draw': function(display, sprites) {
                   this.x = this.c * this.w;
                   this.y = this.l * this.h;
                   display.blit(sprites, 
                                new gamejs.Rect(this.x, this.y, this.w, this.h),
                                new gamejs.Rect(this.sprites[this.pos].x, this.sprites[this.pos].y, this.w, this.h));
               }
             };

var ROBOT_AI = { 'anim': 0,
                 'w': SPRITE_WIDTH,
                 'h': SPRITE_HEIGHT,
                 'c': 0,
                 'l': 0,
                 'x': this.c*SPRITE_WIDTH,
                 'y': this.l*SPRITE_HEIGHT,
                 'v': 3,  /* 3 px/s */
                 'pos': 'wait0',
                 'dir': 'left',
                 'sprites': {
                     'wait0': { 'x': 8*SPRITE_WIDTH, 'y': 0*SPRITE_HEIGHT },
                     'wait1': { 'x': 8*SPRITE_WIDTH, 'y': 0*SPRITE_HEIGHT },
                     'walk_right0': { 'x': 0*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'walk_right1': { 'x': 1*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'walk_right2': { 'x': 2*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'walk_left0': { 'x': 5*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'walk_left1': { 'x': 4*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'walk_left2': { 'x': 3*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'climb0': { 'x': 6*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'climb1': { 'x': 7*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'rope_right0': { 'x': 8*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'rope_right1': { 'x': 9*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'rope_right2': { 'x': 10*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'rope_left0': { 'x': 13*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'rope_left1': { 'x': 12*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'rope_left2': { 'x': 11*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'fall_right': { 'x': 14*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT },
                     'fall_left': { 'x': 15*SPRITE_WIDTH, 'y': 2*SPRITE_HEIGHT }
                 },
                 'transition': {
                     'wait0': { 'left': 'walk_left0', 'right': 'walk_right0',
                                'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                'none': 'wait1',
                                'up': 'climb1', 'down': 'climb1', 'fall': 'fall_right' },
                     'wait1': { 'left': 'walk_left0', 'right': 'walk_right0',
                                'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                'none': 'wait0',
                                'up': 'climb0', 'down': 'climb0', 'fall': 'fall_left' },
                     'walk_right0': { 'left': 'walk_left0', 'right': 'walk_right1',
                                      'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                      'none': 'walk_right0',
                                      'up': 'climb0', 'down': 'climb0', 'fall': 'fall_right' },
                     'walk_right1': { 'left': 'walk_left0', 'right': 'walk_right2',
                                      'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                      'none': 'walk_right1',
                                      'up': 'climb0', 'down': 'climb0', 'fall': 'fall_right' },
                     'walk_right2': { 'left': 'walk_left0', 'right': 'walk_right0',
                                      'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                      'none': 'walk_right2',
                                      'up': 'climb0', 'down': 'climb0', 'fall': 'fall_right' },
                     'walk_left0': { 'left': 'walk_left1', 'right': 'walk_right0',
                                     'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                     'none': 'walk_left0',
                                     'up': 'climb1', 'down': 'climb1', 'fall': 'fall_left' },
                     'walk_left1': { 'left': 'walk_left2', 'right': 'walk_right0',
                                     'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                     'none': 'walk_left1',
                                     'up': 'climb1', 'down': 'climb1', 'fall': 'fall_left' },
                     'walk_left2': { 'left': 'walk_left0', 'right': 'walk_right0',
                                     'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                     'none': 'walk_left2',
                                     'up': 'climb1', 'down': 'climb1', 'fall': 'fall_left' },
                     'climb0': { 'left': 'walk_left0', 'right': 'walk_right0',
                                 'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                 'none': 'climb0',
                                 'up': 'climb1', 'down': 'climb1', 'fall': 'fall_left' },
                     'climb1': { 'left': 'walk_left0', 'right': 'walk_right0',
                                 'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                 'none': 'climb1',
                                 'up': 'climb0', 'down': 'climb0', 'fall': 'fall_right' },
                     'rope_right0': { 'left': 'walk_left0', 'right': 'walk_right1',
                                      'rope_left': 'rope_left0', 'rope_right': 'rope_right1',
                                      'none': 'rope_right0',
                                      'up': 'climb0', 'down': 'fall_right', 'fall': 'fall_right' },
                     'rope_right1': { 'left': 'walk_left0', 'right': 'walk_right2',
                                      'rope_left': 'rope_left0', 'rope_right': 'rope_right2',
                                      'none': 'rope_right1',
                                      'up': 'climb0', 'down': 'fall_right', 'fall': 'fall_right' },
                     'rope_right2': { 'left': 'walk_left0', 'right': 'walk_right0',
                                      'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                      'none': 'rope_right2',
                                      'up': 'climb0', 'down': 'fall_right', 'fall': 'fall_right' },
                     'rope_left0': { 'left': 'walk_left1', 'right': 'walk_right0',
                                     'rope_left': 'rope_left1', 'rope_right': 'rope_right0',
                                     'none': 'rope_left0',
                                     'up': 'climb1', 'down': 'fall_left', 'fall': 'fall_left' },
                     'rope_left1': { 'left': 'walk_left2', 'right': 'walk_right0',
                                     'rope_left': 'rope_left2', 'rope_right': 'rope_right0',
                                     'none': 'rope_left1',
                                     'up': 'climb1', 'down': 'fall_left', 'fall': 'fall_left' },
                     'rope_left2': { 'left': 'walk_left0', 'right': 'walk_right0',
                                     'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                     'none': 'rope_left2',
                                     'up': 'climb1', 'down': 'fall_left', 'fall': 'fall_left' },
                     'fall_right': { 'left': 'walk_left0', 'right': 'walk_right0',
                                     'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                     'none': 'fall_right',
                                     'up': 'climb1', 'down': 'climb1', 'fall': 'fall_right' },
                     'fall_left': { 'left': 'walk_left0', 'right': 'walk_right0',
                                    'rope_left': 'rope_left0', 'rope_right': 'rope_right0',
                                    'none': 'fall_left',
                                    'up': 'climb0', 'down': 'climb0', 'fall': 'fall_left' }
                 },
                 'update': function(msDuration) {
                     /* Updt animation frame */
                     this.anim += msDuration;
                     
                     /* Catch the player */
                     if (this.anim > 1000/this.v) {
                         /* Current pos/floor */
                         var rc = this.c;
                         var rl = this.l;
                         this.dir = 'none';

                         /* In a hole? */
                         if ( MINE.map[this.l][this.c] == HOLE 
                              || MINE.map[this.l][this.c] == HOLE0 
                              || MINE.map[this.l][this.c] == HOLE1 ) {
                             this.dir = 'none';
                         }
                         else if ( MINE.map[this.l][this.c] == HOLE2 ) {
                             this.dir = 'up';  /* exit */
                         }
                         /* Change floor? */
                         else if (this.l > PLAYER.l
                                  && MINE.map[this.l][this.c] == LADDER
                                  && MINE.map[this.l-1][this.c] != BRICK 
                                  && MINE.map[this.l-1][this.c] != CEMENT ) { 
                             this.dir = 'up';
                         }
                         else if (this.l < PLAYER.l
                                  && MINE.map[this.l+1][this.c] != BRICK 
                                  && MINE.map[this.l+1][this.c] != CEMENT ) {
                             this.dir = 'down';
                         }
                         else if (this.c < PLAYER.c) {
                             this.dir = 'right';
                         }
                         else if (this.c > PLAYER.c) {
                             this.dir = 'left';
                         }

                         /* Fall down? */
                         if ( this.l < MINE.lines-1
                              && MINE.map[this.l][this.c] != LADDER
                              && MINE.map[this.l][this.c] != ROPE_LADDER
                              && MINE.map[this.l][this.c] != HOLE 
                              && MINE.map[this.l][this.c] != HOLE0 
                              && MINE.map[this.l][this.c] != HOLE1 
                              && MINE.map[this.l][this.c] != HOLE2                              
                              && MINE.map[this.l+1][this.c] != BRICK
                              && MINE.map[this.l+1][this.c] != CEMENT 
                              && MINE.map[this.l+1][this.c] != LADDER ) {
                             /* Collision detection */
                             var collide = ROBOTS.some(function(r) {
                                 return (rc == r.c && (rl+1) == r.l);
                             });
                             if (!collide) {
                                 this.dir = 'fall';  // fall!!!
                             }
                         }

                         /* Update robot position */
                         switch(this.dir) {
                         case 'left':
                         case 'walk_left':
                         case 'rope_left':
                             /* Can move left? */
                             if ( MINE.map[this.l][this.c-1] == BRICK
                                  || MINE.map[this.l][this.c-1] == CEMENT ) {
                                 this.dir = 'none';  /* no */
                             }
                             else if ( MINE.map[this.l][this.c-1] == ROPE_LADDER ) {
                                 this.dir = 'rope_left';
                             }
                             /* Collision detection */
                             var collide = ROBOTS.some(function(r) {
                                 return (rl == r.l && (rc-1) == r.c);
                             });
                             /* Ok, move left */
                             if (this.dir != 'none' && !collide) {
                                 this.c -= 1;
                             }
                             /* Updt animation frame */
                             this.pos = this.transition[this.pos][this.dir];
                             break;
                         case 'right':
                         case 'walk_right':
                         case 'rope_right':
                             /* Can move right? */
                             if ( MINE.map[this.l][this.c+1] == BRICK
                                  || MINE.map[this.l][this.c+1] == CEMENT ) {
                                 this.dir = 'none';  /* no */
                             }
                             else if ( MINE.map[this.l][this.c+1] == ROPE_LADDER ) {
                                 this.dir = 'rope_right';
                             }
                             /* Collision detection */
                             var collide = ROBOTS.some(function(r) {
                                 return (rl == r.l && (rc+1) == r.c);
                             });
                             /* Ok, move right */
                             if (this.dir != 'none' && !collide) {
                                 this.c += 1;
                             }
                             /* Updt animation frame */
                             this.pos = this.transition[this.pos][this.dir];
                             break;
                         case 'up':
                             /* Can move up? */
                             if ( MINE.map[this.l-1][this.c] == BRICK
                                  || MINE.map[this.l-1][this.c] == CEMENT ) {
                                 this.dir = 'none';  /* no */
                             }
                             /* Collision detection */
                             var collide = ROBOTS.some(function(r) {
                                 return (rc == r.c && (rl-1) == r.l);
                             });
                             /* Ok, move up */
                             if (this.dir != 'none' && !collide) {
                                 this.l -= 1;
                             }
                             /* Updt animation frame */
                             this.pos = this.transition[this.pos][this.dir];
                             break;
                         case 'down':
                             /* Can move down? */
                             if ( MINE.map[this.l+1][this.c] == BRICK
                                  || MINE.map[this.l+1][this.c] == CEMENT ) {
                                 this.dir = 'none';  /* no */
                             }
                             /* Collision detection */
                             var collide = ROBOTS.some(function(r) {
                                 return (rc == r.c && (rl+1) == r.l);
                             });
                             /* Ok, move down */
                             if (this.dir != 'none' && !collide) {
                                 this.l += 1;
                             }
                             /* Updt animation frame */
                             this.pos = this.transition[this.pos][this.dir];
                             break;
                         case 'fall':
                             /* Ok, fall */
                             this.l += 1;
                             if ( MINE.map[this.l][this.c] == HOLE 
                                  || MINE.map[this.l][this.c] == HOLE0 
                                  || MINE.map[this.l][this.c] == HOLE1 ) {
                                 PLAYER.score += 75;
                                 $("#score").text(("0000000" + PLAYER.score).substr(-7));
                             }
                             /* Updt animation frame */
                             this.pos = this.transition[this.pos][this.dir];
                             break;
                         }
                     }

                     if (this.anim > 1000/this.v) {
                         this.anim = this.anim % (1000/this.v);  /* reset current animation counter */
                     }
                 },
                 'draw': function(display, sprites) {
                     this.x = this.c * this.w;
                     this.y = this.l * this.h;
                     display.blit(sprites, 
                                  new gamejs.Rect(this.x, this.y, this.w, this.h),
                                  new gamejs.Rect(this.sprites[this.pos].x, this.sprites[this.pos].y, this.w, this.h));
                 }
               };
var ROBOTS = [];

var GAMEOVER = 0;
var YOUWIN = 1;
var IN_PROGRESS = 2;
var ENDGAME = 3;

var MINE = { 'anim': 0,
             'cols': 28, 'lines': 16,
             'v': 0.4,
             'paused': true,
             'state': IN_PROGRESS,
             'gold': 0,
             'nb_robots': 0,
             'lvl': 1,  // default mine map
             'map': _.cloneDeep(MINE_LEVELS[001]),
             'update': function(msDuration) {
                 /* Updt animation frame */
                 this.anim += msDuration;

                 /* Plug holes */
                 if (this.anim > 1000/this.v) {
                     for (var j=0; j<this.lines; j++) {
                         for (var i=0; i<this.cols; i++) {
                             switch(this.map[j][i]) {
                             case HOLE:
                                 this.map[j][i] = HOLE0;
                                 break;
                             case HOLE0:
                                 this.map[j][i] = HOLE1;
                                 break;
                             case HOLE1:
                                 this.map[j][i] = HOLE2;
                                 break;
                             case HOLE2:
                                 this.map[j][i] = BRICK;
                                 break;
                             }
                         }
                     }
                 }

                 if (this.anim > 1000/this.v) {
                     this.anim = this.anim % (1000/this.v);  /* reset current animation counter */
                 }
             },
             'draw': function(display, sprites) {                     
                 /* Reset screen */
                 display.clear();
                 gamejs.draw.rect(display, "#000", new gamejs.Rect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT), 0);

                 /* Draw mine */
                 for (var j=0; j<this.lines; j++) {
                     for (var i=0; i<this.cols; i++) {
                         switch(MINE.map[j][i]) {
                         case BLANK_SPACE:
                         case HOLE:
                             display.blit(sprites, 
                                          new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                          new gamejs.Rect(0*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                             break;
                         case HOLE0:
                             display.blit(sprites, 
                                          new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                          new gamejs.Rect(0*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                             break;
                         case HOLE1:
                             display.blit(sprites, 
                                          new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                          new gamejs.Rect(11*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                             break;
                         case HOLE2:
                             display.blit(sprites, 
                                          new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                          new gamejs.Rect(12*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                             break;
                         case BRICK:
                             display.blit(sprites, 
                                          new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                          new gamejs.Rect(1*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                             break;
                         case CEMENT:
                             display.blit(sprites, 
                                          new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                          new gamejs.Rect(2*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                             break;
                         case LADDER:
                             display.blit(sprites, 
                                          new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                          new gamejs.Rect(3*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                             break;
                         case ROPE_LADDER:
                             display.blit(sprites, 
                                          new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                          new gamejs.Rect(4*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                             break;
                         case FALSE_BRICK:
                             if ($.urlparam("debug") !== 0) {
                                 display.blit(sprites, 
                                              new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                              new gamejs.Rect(5*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                             }
                             else {
                                 display.blit(sprites, 
                                              new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                              new gamejs.Rect(1*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                             }
                             break;
                         case LADDER_EXIT:
                             if (PLAYER.anim && this.gold == 0) {
                                 this.map[j][i] = LADDER;  /* open the exit */
                             }
                             else {
                                 if ($.urlparam("debug") !== 0) {
                                     display.blit(sprites, 
                                                  new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                                  new gamejs.Rect(6*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                                 }
                                 else {
                                     display.blit(sprites, 
                                                  new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                                  new gamejs.Rect(0*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                                 }
                             }
                             break;
                         case GOLD_CHEST:
                             if (PLAYER.anim == 0) {
                                 this.gold += 1;  /* count remaining gold chest */
                                 $("#gold").text(this.gold);
                             }
                             display.blit(sprites, 
                                          new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                                          new gamejs.Rect(7*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));
                             break;
                         case ROBOT:
                             /*display.blit(sprites, 
                               new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                               new gamejs.Rect(8*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));*/
                             this.map[j][i] = 0;
                             if (PLAYER.anim == 0) {
                                 var robot = _.cloneDeep(ROBOT_AI);
                                 robot.c = i;
                                 robot.l = j;
                                 robot.x = robot.c*SPRITE_WIDTH;
                                 robot.y = robot.l*SPRITE_HEIGHT;
                                 ROBOTS.push(robot);
                                 this.nb_robots += 1;  /* count */
                                 $("#nb_robots").text(this.nb_robots);
                             }
                             break;
                         case LODE_RUNNER:  // player
                             /*display.blit(sprites, 
                               new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 
                               new gamejs.Rect(9*SPRITE_WIDTH, 0*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT));*/
                             this.map[j][i] = 0;    
                             if (PLAYER.anim == 0) {
                                 PLAYER.c = i;
                                 PLAYER.l = j;
                                 PLAYER.x = PLAYER.c*SPRITE_WIDTH;
                                 PLAYER.y = PLAYER.l*SPRITE_HEIGHT;
                                 PLAYER.pos = 'wait0';
                             }
                             break;
                         default:  // error
                             gamejs.draw.rect(display, "#f00", new gamejs.Rect(i*SPRITE_WIDTH, j*SPRITE_HEIGHT, SPRITE_WIDTH, SPRITE_HEIGHT), 0);
                             break;
                         }
                     }
                 }
             }
           };

var CANVAS_WIDTH = MINE.cols * SPRITE_WIDTH;
var CANVAS_HEIGHT = MINE.lines * SPRITE_HEIGHT;


/* Preloads & inits */
gamejs.preload(["./img/sprites.png"]);


/*
 * Main entry point.
 */
function main() {
    $('#version').text("v"+VERSION);

    /* Check Cookies */
    var lrjs_cookie = $.cookie('lrjs');
    if (lrjs_cookie) {
        var lrjs_data = lrjs_cookie.split(',');
        PLAYER.score = +lrjs_data[0];
        PLAYER.men = +lrjs_data[1];
        MINE.lvl = +lrjs_data[2];
    }
    else {  /* register a new one */
        $.cookie('lrjs', '0000000,004,001', { expires: 365 });
        PLAYER.score = 0;
        PLAYER.men = 4;
        MINE.lvl = 1;
    }
    /* Initialize level */
    MINE.map = _.cloneDeep(MINE_LEVELS[MINE.lvl]);
    $("#score").text(("0000000" + PLAYER.score).substr(-7));
    $("#men").text(("000" + PLAYER.men).substr(-3));
    $("#level").text(("000" + MINE.lvl).substr(-3));

    /* Parse URL for options name */
    $.urlparam = function(name) {
        var results = new RegExp("[\\?&amp;]"+name+"=?([^&amp;#]*)").exec(window.location.href);
        return ( results && results[1] ) || 0;
    };

    /* Override level from URL */
    if ($.urlparam("level") != 0) {
        MINE.lvl = +$.urlparam("level").valueOf();
        MINE.map = _.cloneDeep(MINE_LEVELS[MINE.lvl]);
        $("#level").text(("000" + MINE.lvl).substr(-3));
    }

    /* Switch in debug mode? */
    //gamejs.log("debug='"+$.urlparam("debug")+"'");
    if ($.urlparam("debug") !== 0) {
        $("#debug").css("display", "table");
        $("#cheat").css("display", "table");

        /* Populate levels list */
        for (var lv in MINE_LEVELS) {
            if ($("#level").text() == ("000" + lv).substr(-3)) {
                $("#levels").append("<option value=\""+lv+"\" selected=\"1\">"+("000" + lv).substr(-3)+"</option>");
            }
            else {
                $("#levels").append("<option value=\""+lv+"\">"+("000" + lv).substr(-3)+"</option>");
            }
        }
        $("#levels").change(function() {
            /* Initialize selected level */
            MINE.lvl = +$(this).val().valueOf();
            MINE.map = _.cloneDeep(MINE_LEVELS[MINE.lvl]);
            $("#level").text(("000" + MINE.lvl).substr(-3));
            ROBOTS = [];
            MINE.paused = true;
            MINE.nb_robots = 0;
            MINE.gold = 0;
            PLAYER.anim = 0;
            /* Reload score & lives from cookie */
            var lrjs_data = $.cookie('lrjs').split(',');
            PLAYER.score = +lrjs_data[0];
            PLAYER.men = +lrjs_data[1];
            $("#score").text(("0000000" + PLAYER.score).substr(-7));
            $("#men").text(("000" + PLAYER.men).substr(-3));
            /* Save to cookie */
            $.cookie('lrjs', ("0000000"+PLAYER.score).substr(-7)+","+("000"+PLAYER.men).substr(-3)+","+("000"+MINE.lvl).substr(-3), { expires: 365 });
        });
    }

    /* GameJS init */
    var display = gamejs.display.setMode([CANVAS_WIDTH, CANVAS_HEIGHT]);
    var sprites = gamejs.image.load('./img/sprites.png');

    /* Event handling */
    gamejs.onEvent(PLAYER.input.bind(PLAYER));  /* 'this' keyword set to the PLAYER */
    
    /* Game loop: clear display, update & redraw */
    gamejs.onTick(function(msDuration) {
        $("#fps").text(Math.round(1000/msDuration));

        /* Game paused? */
        if (MINE.paused && PLAYER.anim > 0) {
            switch(MINE.state) {
            case GAMEOVER:
                display.blit(new gamejs.font.Font("10px Sans-serif", "#f00").render("Game Over", "#fff"));
                break;
            case YOUWIN:
                display.blit(new gamejs.font.Font("10px Sans-serif", "#0f0").render("You Win", "#fff"));
                break;
            case ENDGAME:
                display.blit(new gamejs.font.Font("10px Sans-serif", "#0f0").render("End Game", "#fff"));
                break;
            default:
                display.blit(new gamejs.font.Font("10px Sans-serif", "#ff0").render("Paused", "#000"));
                break;
            }
            return;  /* pause */
        }

        /* Draw mine */
        MINE.update(msDuration);
        MINE.draw(display, sprites);

        /* Anim robots */
        ROBOTS.forEach(function(robot) {
            robot.update(msDuration);
            robot.draw(display, sprites);        
        });

        /* Check gameover & end level */
        var gameover = ROBOTS.some(function(r) {
            return (PLAYER.c == r.c && PLAYER.l == r.l);
        });
        if (gameover) {
            display.blit(new gamejs.font.Font("10px Sans-serif", "#f00").render("Game Over", "#fff"));
            /* Reinitialize level */
            MINE.map = _.cloneDeep(MINE_LEVELS[MINE.lvl]);
            $("#level").text(("000" + MINE.lvl).substr(-3));
            ROBOTS = [];
            MINE.paused = true;
            MINE.state = GAMEOVER;
            MINE.nb_robots = 0;
            MINE.gold = 0;
            PLAYER.anim = 0;
            /* Reload score from cookie */
            var lrjs_data = $.cookie('lrjs').split(',');
            PLAYER.score = +lrjs_data[0];
            if (PLAYER.men > 0) {
                PLAYER.men -= 1;  /* but lose a live */
            }
            $("#score").text(("0000000" + PLAYER.score).substr(-7));
            $("#men").text(("000" + PLAYER.men).substr(-3));
            /* Save to cookie */
            $.cookie('lrjs', ("0000000"+PLAYER.score).substr(-7)+","+("000"+PLAYER.men).substr(-3)+","+("000"+MINE.lvl).substr(-3), { expires: 365 });
            return;  /* the thief was caught, retry */
        }
        if (MINE.gold == 0 && PLAYER.l < 0) {
            display.blit(new gamejs.font.Font("10px Sans-serif", "#0f0").render("You Win", "#fff"));
            PLAYER.score += 1500;
            $("#score").text(("0000000" + PLAYER.score).substr(-7));
            /* Last level? */
            if (MINE.lvl == MINE_LEVELS.length-1) {
                /* You end the game */
                MINE.paused = true;
                MINE.state = ENDGAME;
                /* Save to cookie */
                $.cookie('lrjs', ("0000000"+PLAYER.score).substr(-7)+","+("000"+PLAYER.men).substr(-3)+","+("000"+MINE.lvl).substr(-3), { expires: 365 });
                /* TODO: restart from level 1 or freeze at 150 ? */
            }
            else {
                /* Initialize new level */
                MINE.lvl = +MINE.lvl + 1;
                MINE.map = _.cloneDeep(MINE_LEVELS[MINE.lvl]);
                $("#level").text(("000" + MINE.lvl).substr(-3));
                ROBOTS = [];
                MINE.paused = true;
                MINE.state = YOUWIN;
                MINE.nb_robots = 0;
                MINE.gold = 0;
                PLAYER.anim = 0;
                PLAYER.l = 0;
                /* Save to cookie */
                $.cookie('lrjs', ("0000000"+PLAYER.score).substr(-7)+","+("000"+PLAYER.men).substr(-3)+","+("000"+MINE.lvl).substr(-3), { expires: 365 });
            }
            return;  /* you win, do next level */
        }

        /* Anim player */
        PLAYER.update(msDuration);
        PLAYER.draw(display, sprites);
    });
};

gamejs.ready(main);
